@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005-2008 by the FIFE team                              *
- *   http://www.fifengine.de                                               *
+ *   Copyright (C) 2005-2013 by the FIFE team                              *
+ *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
  *   FIFE is free software; you can redistribute it and/or                 *
@@ -401,7 +401,7 @@ namespace FIFE {
 
 		SDL_ActiveEvent actevt = event.active;
 		std::vector<Command*> commands;
-		
+
 		if (actevt.state & SDL_APPMOUSEFOCUS) {
 			Command* cmd = new Command();
 			if (actevt.gain) {
@@ -643,11 +643,11 @@ namespace FIFE {
 		return m_mousesensitivity;
 	}
 
-	void EventManager::setMouseAcceleration(bool acceleration) {
+	void EventManager::setMouseAccelerationEnabled(bool acceleration) {
 		m_acceleration = acceleration;
 	}
 
-	bool EventManager::getMouseAcceleration() const {
+	bool EventManager::isMouseAccelerationEnabled() const {
 		return m_acceleration;
 	}
 }
