@@ -1,5 +1,5 @@
 /***************************************************************************
- *   Copyright (C) 2005-2011 by the FIFE team                              *
+ *   Copyright (C) 2005-2013 by the FIFE team                              *
  *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
@@ -39,6 +39,9 @@
 #include "sdlimage.h"
 
 namespace FIFE {
+	/** Logger to use for this source file.
+	 *  @relates Logger
+	 */
 	static Logger _log(LM_VIDEO);
 
 	SDLImage::SDLImage(IResourceLoader* loader):
@@ -410,6 +413,13 @@ namespace FIFE {
 		if( m_finalized ) {
 			return;
 		}
+		// ultimate possibility to load the image
+		// is used e.g. in case a cursor or gui image is freed even if there is a reference 
+		if (!m_surface) {
+			assert(m_state == IResource::RES_NOT_LOADED);
+			load();
+			assert(m_surface);
+		}
 		m_finalized = true;
 		SDL_Surface *old_surface = m_surface;
 		Uint32 key = SDL_MapRGB(m_surface->format, m_colorkey.r, m_colorkey.g, m_colorkey.b);
@@ -545,7 +555,7 @@ namespace FIFE {
 			return SDL_DisplayFormatAlpha(src);
 		}
 
-		SDL_Surface *dst = SDL_CreateRGBSurface(src->flags & ~(SDL_SRCALPHA) | SDL_SWSURFACE,
+		SDL_Surface *dst = SDL_CreateRGBSurface((src->flags & ~(SDL_SRCALPHA)) | SDL_SWSURFACE,
 		                                        src->w, src->h,
 		                                        src->format->BitsPerPixel,
 		                                        src->format->Rmask,  src->format->Gmask,
