@@ -1,6 +1,6 @@
 /***************************************************************************
- *   Copyright (C) 2005-2008 by the FIFE team                              *
- *   http://www.fifengine.de                                               *
+ *   Copyright (C) 2005-2013 by the FIFE team                              *
+ *   http://www.fifengine.net                                              *
  *   This file is part of FIFE.                                            *
  *                                                                         *
  *   FIFE is free software; you can redistribute it and/or                 *
@@ -42,22 +42,22 @@
 #include "model/structures/layer.h"
 #include "model/structures/location.h"
 #include "model/structures/map.h"
+#include "model/structures/cell.h"
+#include "model/structures/cellcache.h"
 #include "video/opengl/fife_opengl.h"
 
 #include "view/camera.h"
 #include "view/visual.h"
+#include "cellrenderer.h"
 #include "instancerenderer.h"
 
 #include "video/opengle/gleimage.h"
 
 
-namespace {
-	uint32_t scale(uint32_t val, double factor) {
-		return static_cast<uint32_t>(ceil(static_cast<double>(val) * factor));
-	}
-}
-
 namespace FIFE {
+	/** Logger to use for this source file.
+	 *  @relates Logger
+	 */
 	static Logger _log(LM_VIEWVIEW);
 
 	class InstanceRendererDeleteListener : public InstanceDeleteListener {
@@ -88,6 +88,7 @@ namespace FIFE {
 		r(0),
 		g(0),
 		b(0),
+		a(128),
 		dirty(false),
 		curimg(NULL),
 		renderer(r) {
@@ -213,10 +214,10 @@ namespace FIFE {
 			}
 		}
 
-		// This values comes from glOrtho settings
-		static const double global_z_min = -100.0;
+		// These values come from glOrtho settings
 		static const double global_z_max = 100.0;
 #if 0
+		static const double global_z_min = -100.0;
 		static const double depth_range = fabs(global_z_min - global_z_max);
 
 		// This is how much depth we can sacrifice for given layer
@@ -260,8 +261,8 @@ namespace FIFE {
 					InstanceToColoring_t::iterator coloring_it = m_instance_colorings.find(instance);
 					const bool coloring = coloring_it != m_instance_colorings.end();
 					if (coloring) {
-						uint8_t rgb[3] = { coloring_it->second.r, coloring_it->second.g, coloring_it->second.b };
-						vc.image->renderZ(vc.dimensions, vertexZ, 255, false, rgb);
+						uint8_t rgba[4] = { coloring_it->second.r, coloring_it->second.g, coloring_it->second.b, coloring_it->second.a };
+						vc.image->renderZ(vc.dimensions, vertexZ, 255, false, rgba);
 					}
 
 					if (outline || coloring) {
@@ -360,6 +361,10 @@ namespace FIFE {
 				}
 			}
 		}
+		// Fixme
+		CellRenderer* cr = dynamic_cast<CellRenderer*>(cam->getRenderer("CellRenderer"));
+		Layer* fow_layer = cr->getFowLayer();
+		bool check_fow = (fow_layer == layer && cr->isEnabledFogOfWar());
 
 		RenderList::iterator instance_it = instances.begin();
 		for (;instance_it != instances.end(); ++instance_it) {
@@ -367,6 +372,15 @@ namespace FIFE {
 			Instance* instance = (*instance_it)->instance;
 			RenderItem& vc = **instance_it;
 
+			if (check_fow) {
+				Cell* cell = layer->getCellCache()->getCell(instance->getLocationRef().getLayerCoordinates());
+				if (cell) {
+					if (cell->getFoWType() != CELLV_REVEALED) {
+						continue;
+					}
+				}
+			}
+
 			if(m_area_layer) {
 				InstanceToAreas_t::iterator areas_it = m_instance_areas.begin();
 				for(;areas_it != m_instance_areas.end(); areas_it++) {
@@ -390,6 +404,10 @@ namespace FIFE {
 							rec.h = infoa.h;
 							if(infoa.instance != instance && vc.dimensions.intersects(rec)) {
 								vc.transparency = 255 - infoa.trans;
+								// dirty hack to reset the transparency on next pump
+								InstanceVisual* visual = instance->getVisual<InstanceVisual>();
+								visual->setVisible(!visual->isVisible());
+								visual->setVisible(!visual->isVisible());
 							}
 						}
 					}
@@ -421,8 +439,8 @@ namespace FIFE {
 						bindColoring(coloring_it->second, vc, cam)->render(vc.dimensions, vc.transparency);
 						m_renderbackend->changeRenderInfos(1, 4, 5, true, false, 0, KEEP, ALWAYS);
 					} else {
-						uint8_t rgb[3] = { coloring_it->second.r, coloring_it->second.g, coloring_it->second.b };
-						vc.image->render(vc.dimensions, vc.transparency, rgb);
+						uint8_t rgba[4] = { coloring_it->second.r, coloring_it->second.g, coloring_it->second.b, coloring_it->second.a };
+						vc.image->render(vc.dimensions, vc.transparency, rgba);
 						m_renderbackend->changeRenderInfos(1, 4, 5, true, false, 0, KEEP, ALWAYS);
 					}
 				}
@@ -496,7 +514,7 @@ namespace FIFE {
 		bool found = false;
 		// create name
 		std::stringstream sts;
-		sts << vc.image.get()->getName() << "," << static_cast<uint32_t>(info.r) << "," << 
+		sts << vc.image.get()->getName() << "," << static_cast<uint32_t>(info.r) << "," <<
 			static_cast<uint32_t>(info.g) << "," << static_cast<uint32_t>(info.b) << "," << info.width;
 		// search image
 		if (ImageManager::instance()->exists(sts.str())) {
@@ -510,14 +528,13 @@ namespace FIFE {
 			found = true;
 		}
 
-			
+
 		// With lazy loading we can come upon a situation where we need to generate outline from
 		// uninitialised shared image
 		if(vc.image->isSharedImage()) {
 			vc.image->forceLoadInternal();
 		}
 
-		SDL_Surface* surface = vc.image->getSurface();
 		SDL_Surface* outline_surface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA,
 			vc.image->getWidth(), vc.image->getHeight(), 32,
 			RMASK, GMASK, BMASK, AMASK);
@@ -602,8 +619,8 @@ namespace FIFE {
 		bool found = false;
 		// create name
 		std::stringstream sts;
-		sts << vc.image.get()->getName() << "," << static_cast<uint32_t>(info.r) << "," << 
-			static_cast<uint32_t>(info.g) << "," << static_cast<uint32_t>(info.b);
+		sts << vc.image.get()->getName() << "," << static_cast<uint32_t>(info.r) << "," <<
+			static_cast<uint32_t>(info.g) << "," << static_cast<uint32_t>(info.b) << "," << static_cast<uint32_t>(info.a);
 		// search image
 		if (ImageManager::instance()->exists(sts.str())) {
 			info.overlay = ImageManager::instance()->getPtr(sts.str());
@@ -624,18 +641,17 @@ namespace FIFE {
 		}
 
 		// not found so we create it
-		SDL_Surface* surface = vc.image->getSurface();
 		SDL_Surface* overlay_surface = SDL_CreateRGBSurface(SDL_SWSURFACE | SDL_SRCALPHA,
 			vc.image->getWidth(), vc.image->getHeight(), 32,
 			RMASK, GMASK, BMASK, AMASK);
 
 		uint8_t r, g, b, a = 0;
-
+		float alphaFactor = static_cast<float>(info.a/255.0);
 		for (int32_t x = 0; x < overlay_surface->w; x ++) {
 			for (int32_t y = 0; y < overlay_surface->h; y ++) {
 				vc.image->getPixelRGBA(x, y, &r, &g, &b, &a);
 				if (a > 0) {
-					Image::putPixel(overlay_surface, x, y, (r + info.r) >> 1, (g + info.g) >> 1, (b + info.b) >> 1, a);
+					Image::putPixel(overlay_surface, x, y, info.r*(1.0-alphaFactor) + r*alphaFactor, info.g*(1.0-alphaFactor) + g*alphaFactor, info.b*(1.0-alphaFactor) + b*alphaFactor, a);
 				}
 			}
 		}
@@ -656,7 +672,7 @@ namespace FIFE {
 		}
 		// mark overlay as not dirty since we created/recreated it here
 		info.dirty = false;
-		
+
 		return info.overlay.get();
 	}
 
@@ -703,11 +719,12 @@ namespace FIFE {
 		}
 	}
 
-	void InstanceRenderer::addColored(Instance* instance, int32_t r, int32_t g, int32_t b) {
+	void InstanceRenderer::addColored(Instance* instance, int32_t r, int32_t g, int32_t b, int32_t a) {
 		ColoringInfo newinfo(this);
 		newinfo.r = r;
 		newinfo.g = g;
 		newinfo.b = b;
+		newinfo.a = a;
 		newinfo.dirty = true;
 
 		// attempts to insert the element into the coloring map
@@ -721,11 +738,12 @@ namespace FIFE {
 			// already exists in the map so lets just update its coloring info
 			ColoringInfo& info = insertiter.first->second;
 
-			if (info.r != r || info.g != g || info.b != b) {
+			if (info.r != r || info.g != g || info.b != b || info.a != a) {
 				// only update the coloring info if its changed since the last call
 				info.r = r;
 				info.b = b;
 				info.g = g;
+				info.a = a;
 				info.dirty = true;
 			}
 		} else {
@@ -762,6 +780,11 @@ namespace FIFE {
 			// the insertion did not happen because the instance
 			// already exists in the map so lets just update its area info
 			AreaInfo& info = insertiter.first->second;
+			info.groups = groups;
+			info.w = w;
+			info.h = h;
+			info.trans = trans;
+			info.front = front;
 		} else {
 			std::pair<InstanceToEffects_t::iterator, bool> iter = m_assigned_instances.insert(std::make_pair(instance, AREA));
 			if (iter.second) {
@@ -928,7 +951,7 @@ namespace FIFE {
 			ImagesToCheck_t::iterator it = m_check_images.begin();
 			for (; it != m_check_images.end(); ++it) {
 				if (it->image.get()->getName() == image.get()->getName()) {
-					return;					
+					return;
 				}
 			}
 			s_image_entry entry;
