@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # ####################################################################
-#  Copyright (C) 2005-2010 by the FIFE team
+#  Copyright (C) 2005-2013 by the FIFE team
 #  http://www.fifengine.net
 #  This file is part of FIFE.
 #
@@ -40,27 +40,30 @@ from fife.extensions.serializers.simplexml import SimpleXMLSerializer
 
 SETTINGS_GUI_XML="""\
 <Window name="Settings" title="Settings">
-	<Label text="Settings menu!" />
-	<HBox>
-		<VBox>
-			<Label text="Resolution:" />
-			<Label text="Renderer:" />
-			<Label text="Light Model:" />
-		</VBox>
-		<VBox min_size="120,60">
-			<DropDown name="screen_resolution" min_size="120,0" />
-			<DropDown name="render_backend" min_size="120,0" />
-			<DropDown name="lighting_model" min_size="120,0" />
-		</VBox>
-	</HBox>
-	<CheckBox name="enable_fullscreen" text="Use the full screen mode" />
-	<CheckBox name="enable_sound" text="Enable sound" />
-	<HBox>
+	<VBox>
 		<Spacer />
-		<Button name="cancelButton" text="Cancel" />
-		<Button name="okButton" text="Ok" />
-		<Button name="defaultButton" text="Defaults" />
-	</HBox>
+		<VBox>
+			<HBox>
+				<Label text="Resolution:" />
+				<ListBox name="screen_resolution" min_size="120,0" />
+			</HBox>
+			<HBox>
+				<Label text="Renderer:" />
+				<ListBox name="render_backend" min_size="120,0" />
+			</HBox>
+			<HBox>
+				<Label text="Light Model:" />
+				<ListBox name="lighting_model" min_size="120,0" />
+			</HBox>
+		</VBox>
+		<CheckBox name="enable_fullscreen" text="Use the full screen mode" />
+		<CheckBox name="enable_sound" text="Enable sound" />
+		<HBox>
+			<Button name="cancelButton" text="Cancel" />
+			<Button name="okButton" text="Ok" />
+			<Button name="defaultButton" text="Defaults" />
+		</HBox>
+	</VBox>
 </Window>
 """
 
@@ -553,11 +556,16 @@ class Setting(object):
 				"""
 				value = self.get(entry.module, entry.name)
 				
-				if type(entry.initialdata) is list:
+				if isinstance(entry.initialdata, list):
 					try:
 						value = entry.initialdata.index(value)
 					except ValueError:
 						raise ValueError("\"" + str(value) + "\" is not a valid value for " + entry.name + ". Valid options: " + str(entry.initialdata))
+				elif isinstance(entry.initialdata, dict):
+					try:
+						value = entry.initialdata.keys().index(value)
+					except ValueError:
+						raise ValueError("\"" + str(value) + "\" is not a valid value for " + entry.name + ". Valid options: " + str(entry.initialdata.keys()))
 				entry.initializeWidget(widget, value)
 
 	def applySettings(self):
@@ -572,8 +580,10 @@ class Setting(object):
 				
 				# If the data is a list we need to get the correct selected data
 				# from the list. This is needed for e.g. dropdowns or listboxs
-				if type(entry.initialdata) is list:
+				if isinstance(entry.initialdata, list):
 					data = entry.initialdata[data]
+				elif isinstance(entry.initialdata, dict):
+					data = entry.initialdata.keys()[data]
 
 				# only take action if something really changed
 				if data != self.get(entry.module, entry.name):
@@ -667,13 +677,14 @@ class SettingEntry(object):
 	def initializeWidget(self, widget, currentValue):
 		"""Initialize the widget with needed data"""
 		if self._initialdata is not None:
-			widget.setInitialData(self._initialdata)
+			if isinstance(self._initialdata, dict):
+				widget.setInitialData(self._initialdata.values())
+			else:
+				widget.setInitialData(self._initialdata)
 		widget.setData(currentValue)
 
 	def onApply(self, data):
-		"""Implement actions that need to be taken when the setting is changed
-		here.
-		"""
+		"""Implement actions that need to be taken when the setting is changed here. """
 		if self._applyfunction is not None:
 			self._applyfunction(data)
 
