@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 
 # ####################################################################
-#  Copyright (C) 2005-2011 by the FIFE team
+#  Copyright (C) 2005-2013 by the FIFE team
 #  http://www.fifengine.net
 #  This file is part of FIFE.
 #
@@ -222,7 +222,7 @@ class Widget(object):
 		raise RuntimeError("No implementation of clone method for %s" % self.__class__)
 			      
 
-	def execute(self,bind):
+	def execute(self, bind, focus=None):
 		"""
 		Execute a dialog synchronously.
 
@@ -234,6 +234,8 @@ class Widget(object):
 		The widget will be shown before execution and hidden afterwards.
 		You can only execute root widgets.
 
+		@param focus: name of child widget which should have focus. Defaults to main widget.
+
 		Note: This feature is not tested well, and the API will probably
 		change. Otherwise have fun::
 		  # Okay this a very condensed example :-)
@@ -251,8 +253,12 @@ class Widget(object):
 				self.hide()
 			self.findChild(name=name).capture( _quitThisDialog , group_name = "__execute__" )
 		self.show()
-		self.is_focusable = True
-		self.requestFocus()
+		if focus and self.findChild(name=focus):
+			self.findChild(name=focus).is_focusable = True
+			self.findChild(name=focus).requestFocus()
+		else:
+			self.is_focusable = True
+			self.requestFocus()
 		return get_manager().mainLoop()
 
 	def requestFocus(self):
@@ -288,8 +294,8 @@ class Widget(object):
 		It might be useful to check out L{tools.callbackWithArguments}.
 
 		@param callback: Event callback - may accept keyword arguments event and widget.
-		@paran event_name: The event to capture - may be one of L{events.EVENTS} and defaults to "action"
-		@paran group_name: Event group.
+		@param event_name: The event to capture - may be one of L{events.EVENTS} and defaults to "action"
+		@param group_name: Event group.
 
 		Event groups are used to have different B{channels} which don't interfere with each other.
 		For derived widgets that need to capture events it's advised to use the group_name 'widget'.
@@ -371,7 +377,7 @@ class Widget(object):
 		If you call adaptLayout on the Label the layout from the VBox 2
 		will get recalculated, while the VBox 1 stays untouched.
 
-		@param recurse Pass False here to force the layout to start from
+		@param recurse: Pass False here to force the layout to start from
 		this widget.
 		"""
 		widget = self
@@ -423,7 +429,7 @@ class Widget(object):
 		their name. This will contain only Widgets which have
 		a name different from "__unnamed__" (which is the default).
 		
-		@param include_unnamed Defaults to false. If this is true unnamed widgets are added, too.
+		@param include_unnamed: Defaults to false. If this is true unnamed widgets are added, too.
 		
 		The values are lists of widgets, so not only unique names
 		are handled correctly.
