@@ -1,7 +1,7 @@
 # -*- coding: utf-8 -*-
 # ####################################################################
-#  Copyright (C) 2005-2010 by the FIFE team
-#  http://www.fifengine.de
+#  Copyright (C) 2005-2013 by the FIFE team
+#  http://www.fifengine.net
 #  This file is part of FIFE.
 #
 #  FIFE is free software; you can redistribute it and/or
@@ -99,8 +99,8 @@ class XMLMapLoader(object):
 		
 		@type	location:	object
 		@param	location:	path to a map file as a fife.ResourceLocation
-		@type	map:	object
-		@return	map:	FIFE map object
+		@return	FIFE map object
+		@rtype	object
 		"""
 		start_time = time.time()
 		self.source = location
@@ -115,13 +115,13 @@ class XMLMapLoader(object):
 
 	def parse_map(self, mapelt):
 		""" start parsing the xml structure and
-		    call submethods for turning found tags
-		    into FIFE objects and create the map
+		call submethods for turning found tags
+		into FIFE objects and create the map
 		
 		@type	mapelt:	object
 		@param	mapelt:	ElementTree root
-		@type	map:	object
-		@return	map:	FIFE map object		
+		@return	FIFE map object
+		@rtype	object
 		"""
 		if not mapelt:
 			self._err('No <map> element found at top level of map file definition.')
@@ -160,8 +160,8 @@ class XMLMapLoader(object):
 		
 		@type	mapelt:	object
 		@param	mapelt:	ElementTree root
-		@type	map:	object
-		@map	map:	FIFE map object			
+		@return	FIFE map object			
+		@rtype	object
 		"""
 		parsedImports = {}
 
@@ -203,7 +203,7 @@ class XMLMapLoader(object):
 		@type	mapelt:	object
 		@param	mapelt:	ElementTree root
 		@type	map:	object
-		@map	map:	FIFE map object			
+		@param	map:	FIFE map object			
 		"""		
 		if self.callback is not None:		
 			tmplist = mapelt.findall('layer')
@@ -225,6 +225,9 @@ class XMLMapLoader(object):
 			pathing = layer.get('pathing')
 			transparency = layer.get('transparency')
 			
+			layer_type = layer.get('layer_type')
+			layer_type_id = layer.get('layer_type_id')
+			
 			if not x_scale: x_scale = 1.0
 			if not y_scale: y_scale = 1.0
 			if not rotation: rotation = 0.0
@@ -259,12 +262,16 @@ class XMLMapLoader(object):
 			strgy = fife.CELL_EDGES_ONLY
 			if pathing == "cell_edges_and_diagonals":
 				strgy = fife.CELL_EDGES_AND_DIAGONALS
-			if pathing == "freeform":
-				strgy = fife.FREEFORM
 
 			layer_obj.setPathingStrategy(strgy)
 			layer_obj.setLayerTransparency(transparency)
 
+			if layer_type:
+				if layer_type == 'walkable':
+					layer_obj.setWalkable(True)
+				elif layer_type == 'interact' and layer_type_id:
+					layer_obj.setInteract(True, layer_type_id)
+
 			self.parse_instances(layer, layer_obj)
 			
 			if self.extensions['lights']:
@@ -276,6 +283,18 @@ class XMLMapLoader(object):
 				i += 1
 				self.callback(self.msg['layer'] % str(_id), float( i / float(len(tmplist)) * 0.25 + 0.5 ) )
 
+
+		layers = map.getLayers()
+		for l in layers:
+			if l.isInteract():
+				walk_layer = map.getLayer(l.getWalkableId())
+				if walk_layer:
+					walk_layer.addInteractLayer(l);
+				
+		for l in layers:
+			if l.isWalkable():
+				l.createCellCache()
+
 		# cleanup
 		if self.callback is not None:
 			del tmplist
@@ -287,7 +306,7 @@ class XMLMapLoader(object):
 		@type	layerelt:	object
 		@param	layerelt:	ElementTree layer branch
 		@type	layer:	object
-		@map	layer:	FIFE layer object
+		@param	layer:	FIFE layer object
 		"""
 		_LIGHT_DEFAULT_BLENDING_SRC = -1
 		_LIGHT_DEFAULT_BLENDING_DST = -1
@@ -422,7 +441,7 @@ class XMLMapLoader(object):
 		@type	layerelt:	object
 		@param	layerelt:	ElementTree layer branch
 		@type	layer:	object
-		@map	layer:	FIFE layer object
+		@param	layer:	FIFE layer object
 		"""	
 		# to be continued		
 		pass
@@ -433,7 +452,7 @@ class XMLMapLoader(object):
 		@type	layerelt:	object
 		@param	layerelt:	ElementTree layer branch
 		@type	layer:	object
-		@map	layer:	FIFE layer object
+		@param	layer:	FIFE layer object
 		"""			
 		instelt = layerelt.find('instances')
 
@@ -518,15 +537,15 @@ class XMLMapLoader(object):
 	def parse_cameras(self, mapelt, map):
 		""" create all cameras and activate them
 		
-			FIXME:
-				- should the cameras really be enabled here?
-				  IMO that's part of the setup within a client
-				  (we just _load_ things here)
+		FIXME:
+			- should the cameras really be enabled here?
+			  IMO that's part of the setup within a client
+			  (we just _load_ things here)
 		
 		@type	mapelt:	object
 		@param	mapelt:	ElementTree root
 		@type	map:	object
-		@map	map:	FIFE map object			
+		@param	map:	FIFE map object			
 		"""				
 		if self.callback:		
 			tmplist = mapelt.findall('camera')
@@ -580,7 +599,7 @@ class XMLMapLoader(object):
 				
 	def create_light_nodes(self, map):
 		""" loop through all preloaded lights and create them
-			according to their data
+		according to their data
 		
 		@type	map:	object
 		@param	map:	FIFE map object
